from oort.rdfview import *
from myapp.ns import *


class Labelled(RdfQuery):
    label = localized(RDFS)
    @property
    def uriEnd(self):
        return self.uri.split('#')[-1].split('/')[-1]

class Titled(Labelled):
    title = localized(DC)
    altTitle = localized(DCT.alternative)

class Named(Labelled):
    name = one(FOAF)
    givenname = one(FOAF)
    surname = one(FOAF)
    @property
    def fullname(self):
        return self.name or self._make_full_name()
    def _make_full_name(self):
        if not self.givenname:
            return self.surname
        elif self.surname:
            return self.givenname + " " + self.surname
        else:
            return self.surname

class Topic(Labelled):
    prefLabel = localized(SKOS)
    altLabel = localized(SKOS)
    @property
    def topicLabel(self):
        return self.prefLabel or self.altLabel or self.label


# NOTE: this is a slow, manual, limited thing.
# It is superfluous if the used graph has inference capabilities.
class AnyLabelled(Titled, Named, Topic):
    projectName = one(DOAP.name)
    @property
    def anyLabel(self):
        return self.anyName or self.anyTitle or self.topicLabel or self.uri
    @property
    def anyName(self):
        return self.fullname or self.projectName
    @property
    def anyTitle(self):
        return self.altTitle or self.title


class ItemInfo(RdfQuery):
    type = one(RDF)
    parentNode = one_where_self_is(APP.subNode) >> Titled
    publisher = one(DC) >> Named

    @selector
    def parentListing(self, graph, lang, _s):
        return [Titled(graph, lang, subj) for subj in
                graph.subjects(APP.useType, self.type)]
    @property
    def parent(self):
        if self.parentNode:
            return self.parentNode
        elif len(self.parentListing) == 1:
            return self.parentListing[0]


class SiteNode(Titled, ItemInfo):
    nodeContent = localized_xml(APP)
    subNodes = each(APP.subNode) >> Titled | Sorter('title')
    subjects = each(DC.subject) >> Labelled


class WebSite(Titled):
    title = localized(DC)
    startNode = one(APP) >> SiteNode
    toc = collection(APP) >> Titled
    @property
    def altTitle(self):
        return self.startNode.altTitle


class ProjectInfo(RdfQuery):
    name = one(DOAP)
    shortdesc = localized(DOAP)


class Person(Named):
    interests = each(FOAF.interest) >> Topic
    projects = each_where_self_is(DOAP.developer) >> ProjectInfo


class TypeListing(SiteNode):
    useType = one(APP) >> RdfQuery
    @selector
    def instances(self, graph, lang, _s):
        return [ AnyLabelled(graph, lang, subject)
            for subject in graph.subjects(RDF.type, self.useType.uri) ]
    instances | Sorter('anyLabel')


class Project(ProjectInfo):
    programmingLanguages = each(DOAP['programming-language'])
    categories = each(DOAP.category) >> Topic
    developers = each(DOAP.developer) >> Person


class FullTopic(Topic):
    referencedBy = each_where_self_is(None) >> AnyLabelled


